#!/bin/sh
#
# TazPkg CGI interface - Manage packages via a browser
#
# This CGI interface extensively uses tazpkg to manage packages and have
# its own code for some tasks. Please KISS, it is important and keep speed
# in mind. Thanks, Pankso.
#
# (C) 2011-2014 SliTaz GNU/Linux - BSD License
#

. /lib/libtaz.sh
. lib/libtazpanel

. /etc/slitaz/slitaz.conf
. /etc/slitaz/tazpkg.conf

get_config
header

_()  { local T="$1"; shift; printf "$(gettext "$T")" "$@"; echo; }
_n() { local T="$1"; shift; printf "$(gettext "$T")" "$@"; }
_p() {
	local S="$1" P="$2" N="$3"; shift; shift; shift;
	printf "$(ngettext "$S" "$P" "$N")" "$@"; }


# xHTML 5 header with special side bar for categories.
TITLE=$(TEXTDOMAIN='tazpkg'; _ 'TazPanel - Packages')
xhtml_header | sed 's/id="content"/id="content-sidebar"/'

export TEXTDOMAIN='tazpkg'

pkg_info_link()
{
	echo "<a class=\"$2\" href=\"?info=${1//+/%2B}\">$1</a>" | sed 's| class=""||'
}


# Display localized short description

i18n_desc() {
	for L in $LANG ${LANG%%_*}; do
		if [ -e "$PKGS_DB/packages-desc.$L" ]; then
			LOCDESC=$(awk -F$'\t' -vp=$1 '{if ($1 == p) print $2}' $PKGS_DB/packages-desc.$L)
			if [ -n "$LOCDESC" ]; then
				SHORT_DESC="$LOCDESC"
				break
			fi
		fi
	done
}


# We need packages information for list and search

parse_packages_desc() {
	IFS="|"
	cut -f 1,2,3,5 -d "|" | while read PACKAGE VERSION SHORT_DESC WEB_SITE
	do
		class=pkg; [ -d $INSTALLED/${PACKAGE% } ] && class=pkgi
		i18n_desc $PACKAGE
		cat << EOT
<tr>
<td><input type="checkbox" name="pkg" value="$PACKAGE">$(pkg_info_link $PACKAGE $class)</td>
<td>$VERSION</td>
<td>$SHORT_DESC</td>
<td><a class="w" href="$WEB_SITE"></a></td>
</tr>
EOT
	done
	unset IFS
}


parse_packages_info() {
	IFS=$'\t'
	while read PACKAGE VERSION CATEGORY SHORT_DESC WEB_SITE TAGS SIZES DEPENDS; do
		class=pkg; grep -q "^$PACKAGE$'\t'" $PKGS_DB/installed.info && class=pkgi
		i18n_desc $PACKAGE
		cat << EOT
<tr>
<td><input type="checkbox" name="pkg" value="$PACKAGE">$(pkg_info_link $PACKAGE $class)</td>
<td>$VERSION</td>
<td>$SHORT_DESC</td>
<td><a class="w" href="$WEB_SITE"></a></td>
</tr>
EOT
	done
	unset IFS
}


# Show button
show_button() {
	for button in $@; do
		case $button in
		recharge)     img='tp-recharge'; label="$(_ 'Recharge list')" ;;
		up)           img='tp-up';       label="$(_ 'Check upgrades')" ;;
		list)         img='tp-list';     label="$(_ 'My packages')" ;;
		tags)         img='tp-tag';      label="$(_ 'Tags')" ;;
		linkable)     img='tp-link';     label="$(_ 'Linkable packages')" ;;
		admin)        img='tp-admin';    label="$(_ 'Administration')" ;;
		*Install*nf*) img='tp-pkg-ins';  label="$(_ 'Install (Non Free)')" ;;
		*Install*)    img='tp-pkg-ins';  label="$(_ 'Install')" ;;
		*Remove*)     img='tp-pkg-rem';  label="$(_ 'Remove')" ;;
		*Link*)       img='tp-link';     label="$(_ 'Link')" ;;
		*Block*)      img='tp-block';    label="$(_ 'Block')" ;;
		*Unblock*)    img='tp-unblock';  label="$(_ 'Unblock')" ;;
		*Repack*)     img='tp-repack';   label="$(_ 'Repack')" ;;
		*saveconf*)   img='tp-save';     label="$(_ 'Save configuration')" ;;
		*listconf*)   img='tp-list';     label="$(_ 'List configuration files')" ;;
		*quickcheck*) img='tp-check';    label="$(_ 'Quick check')" ;;
		*fullcheck*)  img='tp-check';    label="$(_ 'Full check')" ;;
		*clean*)      img='tp-remove';   label="$(_ 'Clean')" ;;
		*setlink*)    img='tp-link';     label="$(_ 'Set link')" ;;
		*removelink*) img='tp-remove';   label="$(_ 'Remove link')" ;;
		*add-mirror)  img='tp-add';      label="$(_n 'Add mirror')" ;;
		*add-repo)    img='tp-add';      label="$(_n 'Add repository')" ;;
		esac
		cat << EOT
	<button class="button" type="submit" name="${button%%=*}" value="${button#*=}">
		<img src="$IMAGES/$img.png"/>$label
	</button>
EOT
	done
}



#
# xHTML functions
#


# ENTER will search but user may search for a button, so put one.

search_form() {
	[ -z "$repo" ] && repo="$(GET repo)"
	[ -z "$repo" ] && repo="Any"
	cat << EOT
<form method="get" action="">
	<div class="search"><input
		type="search" name="search" results="5" autosave="pkgsearch" autocomplete="on"><input
		type="submit" value="$(_n 'Search')"><input
		type="submit" value="$(_n 'Files')" name="files">
	</div>
</form>
EOT
}


table_head() {
	cat << EOT
<table class="zebra pkglist">
	<thead>
		<tr>
			<td>$(_ 'Name')</td>
			<td>$(_ 'Version')</td>
			<td>$(_ 'Description')</td>
			<td>$(_ 'Web')</td>
		</tr>
	</thead>
	<tbody>
EOT
}


sidebar() {
	repo=$(COOKIE repo); repo=${repo:-Public}; [ -n "$(GET repo)" ] && repo=$(GET repo)
	  my=$(COOKIE my);     my=${my:-my};       [ -n "$(GET my)" ]   &&   my=$(GET my)
	 cat=$(COOKIE cat);   cat=${cat:-all};     [ -n "$(GET cat)" ]  &&  cat=$(GET cat)

	cat << EOT
<script type="text/javascript">
function setCookie(name) {
	document.cookie = name+"="+document.getElementById(name).value; }
function setValue(name, value) {
	document.getElementById(name).value=value;
	setCookie(name); }
</script>
<form method="post" action="?list">

<div id="sidebar">
	<select id="my" value="$my" onchange="setCookie('my'); this.form.submit()">
		<option value="my">$(_ 'My packages')</option>
		<option value="no">$(_ 'All packages')</option>
	</select>

	<h4>$(_ 'Categories')</h4>

	<div class="select_wrap">
	<select id="cat" size="16" onclick="setCookie('cat'); this.form.submit()">
		<option value="base-system" >$(_ 'base-system')</option>
		<option value="x-window"    >$(_ 'x-window')</option>
		<option value="utilities"   >$(_ 'utilities')</option>
		<option value="network"     >$(_ 'network')</option>
		<option value="games"       >$(_ 'games')</option>
		<option value="graphics"    >$(_ 'graphics')</option>
		<option value="office"      >$(_ 'office')</option>
		<option value="multimedia"  >$(_ 'multimedia')</option>
		<option value="development" >$(_ 'development')</option>
		<option value="system-tools">$(_ 'system-tools')</option>
		<option value="security"    >$(_ 'security')</option>
		<option value="misc"        >$(_ 'misc')</option>
		<option value="meta"        >$(_ 'meta')</option>
		<option value="non-free"    >$(_ 'non-free')</option>
		<option value="all"         >$(_ 'all')</option>
		<option value="extra"       >$(_ 'extra')</option>
	</select>
	<script type="text/javascript">
		setValue('my', "$my")
		setValue('cat', "$cat")
	</script>
	</div>
EOT

	if [ -d $PKGS_DB/undigest ]; then
		cat << EOT
	<h4>$(_ 'Repository')</h4>

	<select id="repo" onchange="setCookie('repo')">
		<option value="Public">$(_ 'Public')</option>
		$(for i in $(ls $PKGS_DB/undigest); do
			echo "<option value=\"$i\">$i</option>"
		done)
		<option value="Any">$(_ 'Any')</option>
	</select>
	<script type="text/javascript">setValue('repo', "$repo")</script>
EOT
	fi
	cat << EOT
	<a href="?tags">$(_ 'All tags...')</a>
	<a href="?cats">$(_ 'All categories...')</a>
</div>
</form>
EOT
}


repo_list() {
	if [ -n "$(ls $PKGS_DB/undigest/ 2> /dev/null)" ]; then
		case "$repo" in
			Public)
				;;
			""|Any)
				for i in $PKGS_DB/undigest/* ; do
					[ -d "$i" ] && echo "$i$1"
				done ;;
			*)
				echo "$PKGS_DB/undigest/$repo$1"
				return ;;
		esac
	fi
	echo "$PKGS_DB$1"
}


repo_name() {
	case "$1" in
		$PKGS_DB)
			echo "Public" ;;
		$PKGS_DB/undigest/*)
			echo ${1#$PKGS_DB/undigest/} ;;
	esac
}


header_repo_name() {
	[ -d $PKGS_DB/undigest ] && [ "$repo" != "Public" ] && \
		echo "<h3>$(_ 'Repository: %s' $(repo_name $1))</h3>"
}


# Print links to the pages

pager() {
	PAGE_SIZE=${PAGE_SIZE:-100}
	[ "$PAGE_SIZE" != "0" ] && \
	awk -F'"' -vpage="$page" -vsize="$PAGE_SIZE" -vnum_lines="$(wc -l < $1)" -vtext="$(_ 'Pages:') " -vurl="?list&amp;page=" '
BEGIN{
	num_pages = int(num_lines / size) + (num_lines % size != 0)
	if (num_pages != 1) printf "<p>%s", text
}
{
	if (num_pages == 1) exit
	r = NR % size
	if (r == 1) {
		p = int(NR / size) + 1
		printf "<a class=\"pages%s\" href=\"%s%s\" title=\"%s\n···\n", p==page?" current":"", url, p, $6
	} else if (r == 0)
		printf "%s\">%s</a> ", $6, int(NR / size)
}
END{
	if (num_pages == 1) exit
	if (r != 0) printf "%s\">%s</a>", $6, int(NR / size) + 1
	print "</p>"
}' $1
}


# Show packages list by category or tag

show_list() {
	PAGE_SIZE=${PAGE_SIZE:-100}
	page=$(GET page); page=${page:-1}
	cached=$(mktemp)
	[ -n "$tag" ] && cat=''
	{
		for L in $LANG ${LANG%%_*}; do
			if [ -e "$PKGS_DB/packages-desc.$L" ]; then
				sed '/^#/d' $PKGS_DB/packages-desc.$L; break
			fi
		done
		[ -e "$i/blocked-packages.list" ] && cat $i/blocked-packages.list
		sed 's|.*|&\ti|' $i/installed.info
		[ "$cat" == 'extra' ] || [ $1 == 'my' ] || cat $i/packages.info
		[ "$cat" == 'extra' ] && sed 's|.*|&\t-\textra\t-\thttp://mirror.slitaz.org/packages/get/&\t-\t-\t-|' $PKGS_DB/extra.list
	} | sort -t$'\t' -k1,1 | sed '/^$/d' | awk -F$'\t' -vc="${cat:--}" -vt="${tag:--}" '
{
	if (PKG && PKG != $1) {
		if (SEL) {
			if (DSCL) DSC = DSCL
			printf "<tr><td><input type=\"checkbox\" name=\"pkg\" value=\"%s\"><a class=\"pkg%s%s\" href=\"?info=%s\">%s</a></td><td>%s</td><td>%s</td><td><a href=\"%s\"></a></td></tr>\n", PKG, INS, BLK, gensub(/\+/, "%2B", "g", PKG), PKG, VER, DSC, WEB
		}
		VER = DSC = WEB = DSCL = INS = BLK = SEL = ""
	}

	PKG = $1
	if (NF == 1) { BLK = "b"; next }
	if (NF == 2) { DSCL = $2; next }
	if (c == "all" || $3 == c || index(" "$6" ", " "t" ")) { SEL = 1 }
	if (SEL) {
		if ($9 == "i") { VER = $2; DSC = $4; WEB = $5; INS = "i"; next}
		if (! INS)     { VER = $2; DSC = $4; WEB = $5 }
	}
}' > $cached

	pager="$(pager $cached)"
	case $PAGE_SIZE in
		0) list="$(cat $cached)";;
		*) list="$(tail -n+$((($page-1)*$PAGE_SIZE+1)) $cached | head -n$PAGE_SIZE)";;
	esac

	if [ "$pager" != "<p>$(_ 'Pages:') </p>" ] && [ -n "${list:1:1}" ]; then
		cat << EOT
$(header_repo_name $i)
$pager
	$(table_head)
		$list
	</tbody></table>
$pager
EOT
	fi
	rm -f $cached
}


# Show links for "info" page

show_info_links() {
	if [ -n "$1" ]; then
		echo -n "<tr><td><b>$2</b></td><td>"
		echo $1 | tr ' ' $'\n' | awk -vt="$3" '{
			printf "<a href=\"?%s=%s\">%s</a>   ", t, gensub(/\+/, "%2B", "g", $1), $1
		}'
		echo "</td></tr>"
	fi
}



#
# Commands
#


case " $(GET) " in


	*\ linkable\ *)
		#
		# List linkable packages.
		#
		search_form; sidebar
		LOADING_MSG=$(_ 'Listing linkable packages...'); loading_msg

		cat << EOT
<h2>$(_ 'Linkable packages')</h2>

<form method="get" action="">
<div id="actions">
	<div class="float-left">$(_ 'Selection:') $(show_button do=Link)</div>
	<div class="float-right">$(show_button recharge up)</div>
</div>
EOT
		table_head
		target=$(readlink $PKGS_DB/fslink)
		for pkg in $(ls $target/$INSTALLED); do
			[ -s $pkg/receipt ] && continue
			. $target/$INSTALLED/$pkg/receipt
			i18n_desc $pkg
			cat << EOT
<tr>
	<td><input type="checkbox" name="pkg" value="$pkg" /><a class="pkg" href="?info=${pkg//+/%2B}">$pkg</a></td>
	<td>$VERSION</td>
	<td>$SHORT_DESC</td>
	<td><a class="w" href="$WEB_SITE"></a></td>
</tr>
EOT
		done
		cat << EOT
		</tbody>
	</table>
</form>
EOT
		;;


	*\ cats\ *)
		#
		# List of all categories.
		#
		search_form; sidebar

		echo "<h2>$(_ 'Categories list')</h2>"

		for pkgsinfo in $(repo_list /packages.info); do
			header_repo_name $(dirname $pkgsinfo)
			cat << EOT
<table class="zebra outbox">
	<thead>
		<tr>
			<td>$(_ 'Category')</td>
			<td>$(_ 'Available packages')</td>
			<td>$(_ 'Installed packages')</td>
		</tr>
	</thead>
	<tbody>
EOT
			{
				awk -F$'\t' '{print $3}' $pkgsinfo | sort | uniq -c
				awk -F$'\t' '{print $3}' $PKGS_DB/installed.info | sed 's|.*|& i|' | sort | uniq -c
			} | sort -k2,2 | awk '
			{
				c [$2] = "."
				if ($3) { i[$2] = $1; } else { m[$2] = $1; }
			}
			END {
				for (n in c) print n, m[n], i[n]
			}' | sort | awk '{
			printf "<tr><td><a href=\"?list&amp;cat=%s\">%s</a></td><td>%d</td><td>%d</td></tr>", $1, $1, $2, $3
			}'
			echo '</tbody></table>'
		done
		;;


	*\ list\ *)
		#
		# List all packages by category.
		#
		search_form; sidebar
		LOADING_MSG="$(_ 'Listing packages...')"; loading_msg

		bcat="<b>$cat</b>"; brepo="<b>$repo</b>"
		case $repo in
			Any)
				case $my in
					my) title="$(_ 'Installed packages of category "%s"' "$bcat")" ;;
					*)  title="$(_ 'All packages of category "%s"' "$bcat")" ;;
				esac ;;
			*)
				case $my in
					my) title="$(_ 'Installed packages of category "%s" in repository "%s"' "$bcat" "$brepo")" ;;
					*)  title="$(_ 'All packages of category "%s" in repository "%s"' "$bcat" "$brepo")" ;;
				esac ;;
		esac

		cat << EOT
<h2>$(_ 'Packages list')</h2>
<p>$title</p>

<form method="get" action="">
<div id="actions">
	<div class="float-left">$(_ 'Selection:')
		$([ "$my" != 'my' ] && show_button do=Install)
		$(show_button do=Remove)
	</div>
	<div class="float-right">$(show_button recharge up)</div>
</div>
EOT
		for i in $(repo_list ""); do
			show_list ${my#no}
		done
		echo '</form>'
		;;


	*\ search\ *)
		#
		# Search for packages. Here default is to search in packages.desc
		# and so get result including packages names and descriptions
		#
		pkg=$(GET search); [ -z "$pkg" ] && xhtml_footer && exit
		cd $PKGS_DB

		search_form; sidebar
		LOADING_MSG="$(_ 'Searching packages...')"; loading_msg

		cat << EOT
<h2>$(_ 'Search packages')</h2>

<form method="get" action="">
<div id="actions">
	<div class="float-left">$(_ 'Selection:'; show_button do=Install do=Remove)
		<a href="$(cat $PANEL/lib/checkbox.js)">$(_ 'Toogle all')</a></div>
	<div class="float-right">$(show_button recharge up)</div>
</div>
EOT
		if [ -n "$(GET files)" ]; then
			cat <<EOT
	<table class="zebra outbox filelist">
	<thead>
		<tr>
			<td>$(_ 'Package')</td>
			<td>$(_ 'File')</td>
		</tr>
	<thead>
	<tbody>
EOT
			lzcat $(repo_list /files.list.lzma) | grep -Ei ": .*$(GET search)" | \
			while read PACKAGE FILE; do
				PACKAGE=${PACKAGE%:}
				class=pkg; [ -d $INSTALLED/$PACKAGE ] && class=pkgi
				cat << EOT
<tr>
	<td><input type="checkbox" name="pkg" value="$PACKAGE">$(pkg_info_link $PACKAGE $class)</td>
	<td>$(echo "$FILE" | sed "s|$pkg|<span class=\"diff-add\">$pkg</span>|g")</td>
</tr>
EOT
			done
		else
			table_head
			awk -F$'\t' 'BEGIN{IGNORECASE = 1}
			$1 $4 ~ /'$pkg'/{print $0}' $(repo_list /packages.info) | parse_packages_info
		fi
		cat << EOT
	</tbody>
	</table>
</form>
EOT
		;;


	*\ recharge\ *)
		#
		# Lets recharge the packages list
		#
		search_form; sidebar
		LOADING_MSG="$(_ 'Recharging lists...')"; loading_msg

		cat << EOT
<h2>$(_ 'Recharge')</h2>

<form method="get" action="">
<div id="actions">
	<div class="float-left">
		<p>$(_ 'Recharge checks for new or updated packages')</p>
	</div>
	<div class="float-right">$(show_button up)</div>
</div>
<div class="wrapper">
<pre>
EOT
		echo $(_ 'Recharging packages list') | log
		tazpkg recharge | filter_taztools_msgs
		cat << EOT
</pre>
</div>
<p>$(_ 'Packages lists are up-to-date. You should check for upgrades now.')</p>
EOT
		;;


	*\ up\ *)
		#
		# Upgrade packages
		#
		search_form; sidebar
		LOADING_MSG="$(_ 'Checking for upgrades...')"; loading_msg

		cat << EOT
<h2>$(_ 'Up packages')</h2>

<form method="get" action="">
<div id="actions">
	<div class="float-left">$(_ 'Selection:'; show_button do=Install do=Remove)
	<button class="button" onclick="$(cat $PANEL/lib/checkbox.js)">$(_ 'Toogle all')</button></div>
	<div class="float-right">$(show_button recharge)</div>
</div>
EOT
		# Ask tazpkg to make "packages.up" file
		tazpkg up --check >/dev/null
		table_head

		for pkg in $(cat $PKGS_DB/packages.up); do
			grep -hs "^$pkg	" $PKGS_DB/packages.info $PKGS_DB/undigest/*/packages.info | parse_packages_info
		done

		echo "</tbody></table></form>"
		;;


	*\ do\ *)
		#
		# Do an action on one or some packages
		#
		search_form; sidebar
		loading_msg

		# Find the command
		cmd=$(echo $(GET do) | tr [:upper:] [:lower:])

		# Find the packages list
		pkgs=$(echo $QUERY_STRING | awk 'BEGIN{RS="&";FS="="}{if($1=="pkg")print $2}')
		pkgs=$(httpd -d "$pkgs")

		# Describe the command
		bpkgs="<b>$pkgs</b>"; opt=''
		case $cmd in
			install) MSG="$(_ 'Installing: %s' "$bpkgs")"; opt=--forced; cmd=get-install ;;
			remove)  MSG="$(_ 'Removing: %s'   "$bpkgs")" ;;
			link)    MSG="$(_ 'Linking: %s'    "$bpkgs")"; opt=$(readlink $PKGS_DB/fslink) ;;
			block)   MSG="$(_ 'Blocking: %s'   "$bpkgs")" ;;
			unblock) MSG="$(_ 'Unblocking: %s' "$bpkgs")" ;;
			repack)  MSG="$(_ 'Repacking: %s'  "$bpkgs")" ;;
		esac

		cat << EOT
<h2>TazPkg: $(GET do)</h2>
<p>$MSG</p>
EOT
		# Do the command for all asked packages
		for pkg in $pkgs; do
			echo '<pre>'
			echo $(_n 'y') | tazpkg $cmd $pkg $opt 2>/dev/null | filter_taztools_msgs
			echo '</pre>'
		done ;;


	*\ info\ *)
		#
		# Packages info
		#
		pkg=$(GET info)
		search_form; sidebar
		LOADING_MSG=$(_ 'Getting package info...'); loading_msg

		cat << EOT
<h2>$(_ 'Package %s' $pkg)</h2>

<form method="get" action="">
<input type="hidden" name="pkg" value="${pkg#get-}"/>
<div id="actions">
EOT

		# Get receipt variables, show Install/Remove buttons
		if [ -d $INSTALLED/$pkg ]; then
			. $INSTALLED/$pkg/receipt
			files=$(wc -l < $INSTALLED/$pkg/files.list)
			show_button do=Remove
		else
			cd $PKGS_DB
			eval "$(awk -F$'\t' -vp=$pkg '
			$1==p{
				printf "PACKAGE=\"%s\"; VERSION=\"%s\"; CATEGORY=\"%s\"; ", $1, $2, $3
				printf "SHORT_DESC=\"%s\"; WEB_SITE=\"%s\"; TAGS=\"%s\"; ", $4, $5, $6
				printf "SIZES=\"%s\"; DEPENDS=\"%s\"", $7, $8
			}' packages.info undigest/*/packages.info)"
			PACKED_SIZE=${SIZES% *}
			UNPACKED_SIZE=${SIZES#* }
			if [ "${pkg#get-}" != "$pkg" ]; then
				show_button "do=Install&amp;nf"
			else
				show_button do=Install
			fi
		fi

		# Show Block/Unblock, and Repack buttons
		if [ -d $INSTALLED/$pkg ]; then
			if grep -qs "^$pkg$" $PKGS_DB/blocked-packages.list; then
				show_button do=Unblock
			else
				show_button do=Block
			fi
			show_button do=Repack
		fi

		# Translate short description
		i18n_desc $pkg

		# Show info table
		cat << EOT
</div>
<table class="zebra summary outbox">
<tbody>
	<tr><td><b>$(_ 'Name')</b></td><td>$PACKAGE</td></tr>
	<tr><td><b>$(_ 'Version')</b></td><td>$VERSION</td></tr>
	<tr><td><b>$(_ 'Category')</b></td><td><a href="?cat=$CATEGORY">$CATEGORY</a></td></tr>
	<tr><td><b>$(_ 'Description')</b></td><td>$SHORT_DESC</td></tr>
	$([ -n "$MAINTAINER" ] && echo "<tr><td><b>$(_ 'Maintainer')</b></td><td>$MAINTAINER</td></tr>")
	$([ -n "$LICENSE" ] && echo "<tr><td><b>$(_ 'License')</b></td><td><a href=\"?license=$pkg\">$LICENSE</a></td></tr>")
	<tr><td><b>$(_ 'Website')</b></td><td><a href="$WEB_SITE" target="_blank">$WEB_SITE</a></td></tr>
	$(show_info_links "$TAGS" "$(_ 'Tags')" 'tag')
	<tr><td><b>$(_ 'Sizes')</b></td><td>$PACKED_SIZE/$UNPACKED_SIZE</td></tr>
	$(show_info_links "$DEPENDS" "$(_ 'Depends')" 'info')
	$(show_info_links "$SUGGESTED" "$(_ 'Suggested')" 'info')
</tbody>
</table>
EOT

		# Show description
		DESC="$(tazpkg desc $pkg)"
		[ -n "$DESC" ] && echo "<pre>$DESC</pre>"

		# Show configuration files list
		CONFIGS="$(tazpkg list-config $pkg | sed 's|\(.*\)|\1 \1|')"
		[ -n "$CONFIGS" ] && echo "<p>$(_ 'Configuration files')</p><pre>$(printf '<a href="index.cgi?file=%s">%s</a>\n' $CONFIGS)</pre>"

		# Show installed files list
		if [ -d $INSTALLED/$pkg ]; then
			cat << EOT
<p>$(_ 'Installed files: %s' $(wc -l < $INSTALLED/$pkg/files.list))</p>

<pre>$(sort $INSTALLED/$pkg/files.list)</pre>
EOT
		else
			cat << EOT
<p>$(_ 'Installed files: %s' ' ')</p>

<pre>
$(lzcat files.list.lzma undigest/*/files.list.lzma 2> /dev/null | awk -vp="$pkg:" '$1==p{print $2}' | sort)
</pre>
EOT
		fi
		;;


	*\ admin\ * )
		#
		# TazPkg configuration page
		#
		cmd=$(GET admin)
		pager="$(GET pager)"; pager=${pager:-$PAGE_SIZE}; pager=${pager:-100}
		mirror="$(GET mirror)"; mirror="${mirror%/}/"
		repository="$PKGS_DB/undigest/$(GET repository)"
		link="$(GET link)"; link=${link%/}
		search_form; sidebar
		loading_msg

		case "$cmd" in
			clean)
				rm -rf $CACHE_DIR/* ;;
			add-mirror)
				echo "$mirror" >> $(GET file) ;;
			rm-mirror)
				sed -i "/^"$(echo $mirror | sed 's|/|\\/|g')"$/d" $(GET file) ;;
			select-mirror)
				tazpkg setup-mirror "${mirror}packages/$(cat /etc/slitaz-release)/" | log ;;
			add-repo)
				mkdir -p $repository
				echo "$mirror" > $repository/mirror
				echo "$mirror" > $repository/mirrors ;;
			rm-repo)
				rm -rf $repository ;;
			setlink)
				[ -d "$link/$INSTALLED" ] && ln -fs $link $PKGS_DB/fslink ;;
			removelink)
				rm -f $PKGS_DB/fslink ;;
			pager)
				TP_CONF=/etc/slitaz/tazpanel.conf
				if [ -z "$PAGE_SIZE" ]; then
					echo -e "\n# Size of packages list page\nPAGE_SIZE=\"$pager\"" >> $TP_CONF
				else
					sed -i "s|PAGE_SIZE=.*|PAGE_SIZE=\"$pager\"|" $TP_CONF
				fi ;;
		esac

		cat << EOT
<h2>$(_ 'Administration')</h2>

<p>$(_ 'TazPkg administration and settings')</p>

<form method="get" action="">
<input type="hidden" name="admin" />

<div id="actions">
	$(show_button action=saveconf action=listconf action=quickcheck action=fullcheck)
</div>
</form>
EOT
		case "$(GET action)" in
			saveconf)
				LOADING_MSG=$(_ 'Creating the package...'); loading_msg
				echo "<pre>"
				cd /tmp
				tazpkg repack-config | filter_taztools_msgs
				echo -n "$(_ 'Path:') "; ls /tmp/config-*.tazpkg
				echo "</pre>" ;;
			listconf)
				echo "<h4>$(_ 'Configuration files')</h4>"
				echo "<ul>"
				tazpkg list-config | while read file; do
					if [ -e $file ]; then
						echo "<li><a href=\"index.cgi?file=$file\">$file</a></li>"
					else
						echo "<li>$file</li>"
					fi
				done
				echo "</ul>" ;;
			quickcheck)
				LOADING_MSG=$(_ 'Checking packages consistency...'); loading_msg
				echo "<pre>"
				tazpkg check
				echo "</pre>" ;;
			fullcheck)
				LOADING_MSG=$(_ 'Full packages check...'); loading_msg
				echo "<pre>"
				tazpkg check --full
				echo "</pre>" ;;
			dvdimage)
				dev=$(POST dvdimage)
				mkdir -p /mnt/packages 2> /dev/null
				echo "<pre>"
				mount -t iso9660 -o loop,ro $dev /mnt/packages &&
				/mnt/packages/install.sh &&
				_ '%s is installed on /mnt/packages' $dev
				echo "</pre>" ;;
			dvdusbkey)
				mkdir -p /mnt/packages 2> /dev/null
				for tag in "LABEL=\"packages-$version\" TYPE=\"iso9660\"" \
					"LABEL=\"sources-$version\" TYPE=\"iso9660\"" ; do
					dev=$(blkid | grep "$tag" | cut -d: -f1)
					[ -n "$dev" ] || continue
					echo "<pre>"
					mount -t iso9660 -o ro $dev /mnt/packages &&
					/mnt/packages/install.sh &&
					_ '%s is installed on /mnt/packages' $dev
					echo "</pre>"
					break
				done ;;
		esac

		cache_files=$(find $CACHE_DIR -name '*.tazpkg' | wc -l)
		cache_size=$(du -sh $CACHE_DIR | cut -f1 | sed 's|\.0||')
		[ "$cache_files" == 0 ] && cache_size="0K"
		mirror=$(cat $PKGS_DB/mirror)
		default_mirror=${mirror%/packages/*}
		cat << EOT
<article>
<h3>$(_ 'Packages cache')</h3>

<form method="get" action="">
	<p>$(_ 'Packages in the cache: %s (%s)' $cache_files $cache_size)
		$(show_button admin=clean)
	</p>
</form>
</article>


<article>
<h3>$(_ 'Current mirror list')</h3>
EOT

		# List mirrors
		version=$(cat /etc/slitaz-release)
		for i in $PKGS_DB/mirrors $PKGS_DB/undigest/*/mirrors; do
			[ -s $i ] || continue
			if [ $i != $PKGS_DB/mirrors ]; then
				echo "<h4>$(_ 'Repository: %s' "$(repo_name $(dirname $i))")</h4>"
			fi
			cat << EOT
<form method="get" action="">
<input type="hidden" name="admin" value="select-mirror"/>
<table class="zebra">
EOT
			while read line; do
				cat << EOT
<tr>
	<td>
		<input type="radio" name="mirror" id="$line" value="$line" onchange="this.form.submit()"
			$([ "$line" == "$default_mirror/" ] && echo -n 'checked="checked"')>
		<label for="$line"><code>$line</code></label></td>
	<td><a class="w" href="$line" target="_blank"></a></td>
	<td><a href="?admin=rm-mirror&amp;mirror=$line&amp;file=$i">
			<img src="$IMAGES/tp-remove.png" title="$(_ 'Delete')"/></a></td>
</tr>
EOT
			done < $i
			cat << EOT
</table>
</form>

<form method="get" action="">
	<p>
		<input type="hidden" name="file" value="$i" />
		<input type="text" name="mirror" size="40" />
		$(show_button admin=add-mirror)
	</p>
</form>
EOT
		done
		echo "</article><article>"
		echo "<h3>$(_ 'Private repositories')</h3>"
		if [ -n "$(ls $PKGS_DB/undigest 2> /dev/null)" ]; then
			echo '<table class="zebra">'
			ls $PKGS_DB/undigest 2> /dev/null | while read repo ; do
				cat <<EOT
	<tr><td><a href="?admin=rm-repo&amp;repository=$repo"><img src="$IMAGES/tp-remove.png" title="$(_ 'Delete')"></a>$repo</td></tr>
EOT
			done
			echo '</table>'
		fi

		cat << EOT
<form method="get" action="">
	<p>$(_ 'Name') <input type="text" name="repository" size="10"/>
	$(_ 'URL:') <input type="text" name="mirror" value="http://">
	$(show_button admin=add-repo)</p>
</form>
</article>


<article>
<h3>$(_ 'Link to another SliTaz installation')</h3>

<p>$(_ "This link points to the root of another SliTaz installation. \
You will be able to install packages using soft links to it.")</p>

<form method="get" action="">
<p>
	<input type="text" name="link" value="$(readlink $PKGS_DB/fslink 2> /dev/null)"/>
	$(show_button admin=setlink admin=removelink)
</p>
</form>
</article>


<article>
<h3 id="dvd">$(_ 'SliTaz packages DVD')</h3>

<p>$(_ "A bootable DVD image of all available packages for the %s version is \
generated every day. It also contains a copy of the website and can be used \
without an internet connection. This image can be installed on a DVD or a USB \
key." $version)</p>

<button class="button" onclick='http://mirror.slitaz.org/iso/$version/packages-$version.iso'>
	<img src="$IMAGES/tp-dl-dvd.png" />$(_ 'Download DVD image')</button>
<button class="button" onclick='?admin&amp;action=dvdusbkey'>
	<img src="$IMAGES/tp-link.png" />$(_ 'Install from DVD/USB key')</button>
<div class="box">
	<form method="post" action='?admin&amp;action=dvdimage'>
	$(_ 'Install from ISO image:')
	<input type="text" name="dvdimage" size="40" value="/root/packages-$version.iso" />
</div>
</form>
</article>


<article>
<h3>$(_ 'Packages list')</h3>

<p>$(_ 'Long list of packages is paginated. Here you can set the page size (default: 100, turning off the pager: 0).')</p>
<form method="get" action="">
	<input type="hidden" name="admin" value="pager" />
	<input type="number" name="pager" value="$pager" min="0" step="10" size="4" />
	<button class="button" type="submit">$(_ 'Set')</button>
</form>
</article>
EOT
		;;


	*\ license\ *)
		#
		# Show licenses for installed packages
		#
		search_form; sidebar

		pkg=$(GET license)
		case $pkg in
			/*)	[ -e $pkg ] && {
				echo "<h2>${pkg#/usr/share/licenses/}</h2>"
				case $pkg in
					*.htm*) cat $pkg ;;
					*)	echo "<pre style=\"white-space: pre-wrap\">"
						cat $pkg | htmlize | sed 's|\([hf]t*t*ps*://[a-zA-Z0-9./_-]*[a-zA-Z0-9/_-]\)|<a href="\1">\1</a>|'
						echo "</pre>"
						;;
				esac
				} ;;
			*)	echo "<h2>$(_ 'Licenses for package %s' $pkg)</h2>"
				OFFLINE=''
				if [ -e "$PKGS_DB/installed/$pkg" ]; then
					for lic in $(grep /usr/share/licenses/ $PKGS_DB/installed/$pkg/files.list); do
						OFFLINE="$OFFLINE	<li><a href=\"?license=$lic\">licenses/<b>${lic#/usr/share/licenses/}</b></a></li>\n"
					done
					echo "\
Apache|Apache-2.0||||http://www.apache.org/licenses/#Artistic|Artistic-2.0||||#\
BSD|BSD-2-Clause||||#BSD3|BSD-3-Clause||||#CC-BY-SA||by-sa/4.0/|||#\
CC-SA||by-sa/4.0/|||#CC-BY-ND||by-nd/4.0/|||#CC-BY-NC-SA||by-nc-sa/4.0/|||#\
CC-BY-NC-ND||by-nc-nd/4.0/|||#CC-BY-NC||by-nc/4.0/|||#CC-BY||by/4.0/|||#\
cc-pd|||||http://creativecommons.org/publicdomain/#CDDL|CDDL-1.0||||#\
CECILL|CECILL-2.1||||#Eclipse|EPL-1.0||||#EPL|EPL-1.0||||#FDL|||fdl||#\
GPL|gpl-license||gpl|gpl.txt|#GPL2|GPL-2.0||old-licenses/gpl-2.0||#\
GPL3|GPL-3.0||gpl|gpl.txt|#ISC|ISC||||#LGPL|lgpl-license||lgpl||#\
LGPL2|||old-licenses/lgpl-2.0||#\
LGPL2.1|LGPL-2.1||old-licenses/lgpl-2.1|lgpl.txt|#LGPL3|LGPL-3.0||lgpl||#\
LPPL|LPPL-1.3c||||#MIT|MIT|||mit.txt|#MPL|MPL-2.0|||mozilla.txt|#\
MPL2|MPL-2.0||||#PublicDomain|||||http://creativecommons.org/publicdomain/#\
QPL|QPL-1.0||||#SIL_OFL|OFL-1.1||||#OFL|OFL-1.1||||#zlib/libpng|Zlib||||" | \
awk -vlicenses="$(. $PKGS_DB/installed/$pkg/receipt; echo "$LICENSE")" \
					-vtext="$(_ '%s license on %s website' %s %s)" \
					-vro="$(_ 'Read online:')" -vrl="$(_ 'Read local:')" \
					-vofflic="$OFFLINE" '
BEGIN{ FS="|"; RS="#"; split(licenses, lic, " "); if (offlic) OFFLINE[0]=offlic }
function link1(u, l, ll, w) {
	return sprintf("\t<li><a href=\"%s%s\">" text "</a></li>", u, l, "<b>" ll "</b>", w) }
function link2(u, l, ll) {
	return sprintf("\t<li><a href=\"%s%s\">%s</b></a></li>", u, l, ll) }
function link_osl(n) {
	return link1("http://opensource.org/licenses/", n, n, "OSL") }
function link_cc(n) {
	split(n, ll, "/")
	return link1("http://creativecommons.org/licenses/", n, ll[1], "Creative Commons") }
function link_gnu(n) {
	split(n, ll, "/")
	return link1("https://www.gnu.org/licenses/", n ".html", ll[2] ? ll[2] : ll[1], "GNU") }
function link_loc(n) {
	return link2("?license=/usr/share/licenses/", n, "licenses/<b>" n "</b>") }
function link_url(n) {
	return link2(n, "", n) }
{
	if ($1 == lic[1] || $1 == lic[2] || $1 == lic[3] || $1 == lic[4]) {
		if ($2) ONLINE[$2]=link_osl($2)
		if ($3) ONLINE[$3]=link_cc($3)
		if ($4) ONLINE[$4]=link_gnu($4)
		if ($5) OFFLINE[$5]=link_loc($5)
		if ($6) OFFLINE[$6]=link_url($6)
	}
}
END{
	if (length(ONLINE)  > 0) { print "<p>" ro "</p>\n<ul>"; for (o in ONLINE)  print ONLINE[o]; print "</ul>"  }
	if (length(OFFLINE) > 0) { print "<p>" rl "</p>\n<ul>"; for (o in OFFLINE) print OFFLINE[o]; print "</ul>" }
}'
				fi ;;
		esac
		;;


	*\ tags\ *)
		#
		# Show tag cloud
		#
		search_form; sidebar

		echo "<h2>$(_ 'Tags list')</h2>"
		brepo="<b>$repo</b>"
		case $repo in
			Any) title="$(_ 'List of tags in all repositories')" ;;
			*)   title="$(_ 'List of tags in repository "%s"' "$brepo")" ;;
		esac
		echo "<p>$title</p><p id=\"tags\">"
		to_read=""
		for i in $(repo_list ""); do
			if [ ! -e $i/packages.info ]; then
				list=installed
			else
				list=packages
			fi
			to_read="$to_read $i/$list.info"
		done
		TAGS="$(awk -F$'\t' '{if($6){print $6}}' $to_read | tr ' ' $'\n' | sort | uniq -c)"
		MAX="$(echo "$TAGS" | awk '{if ($1 > MAX) MAX = $1} END{print MAX}')"
		echo "$TAGS" | awk -vMAX="$MAX" '{
			printf "<a class=\"tag%s\" href=\"?tag=%s\" title=\"%s\">%s</a> ", int($1 * 7 / MAX + 1), $2, $1, $2
		}'
		echo "</p>"
		;;


	*\ tag\ *)
		#
		# Show packages with matching tag
		#
		search_form; sidebar

		tag=$(GET tag)
		cat << EOT
<h2>$(_ 'Tag "%s"' $tag)</h2>

<form method="get" action="">
<div id="actions">
	<div class="float-left">
		$(_ 'Selection:'; show_button do=Install do=Remove)
	</div>
	<div class="float-right">
		$(show_button tags)
	</div>
</div>
EOT
		for i in $(repo_list ""); do
			show_list all
		done
		echo '</form>'
		;;


	*\ blocked\ *)
		#
		# Show blocked packages list
		#
		search_form; sidebar

		cat << EOT
<h2>$(_ 'Blocked packages list')</h2>

<form method="get" action="">
<div id="actions">
	<div class="float-left">$(_ 'Selection:'; show_button do=Unblock)
	</div>
</div>
EOT
		table_head
		for i in $(cat $PKGS_DB/blocked-packages.list); do
			awk -F$'\t' -vp="$i" '{
			if ($1 == p)
				printf "<tr><td><input type=\"checkbox\" name=\"pkg\" value=\"%s\"><a class=\"pkgib\" href=\"?info=%s\">%s</a></td><td>%s</td><td>%s</td><td><a href=\"%s\"></a></td></tr>\n", $1, gensub(/\+/, "%2B", "g", $1), $1, $2, $4, $5
			}' $PKGS_DB/installed.info
		done
		echo '</tbody></table></form>'
		;;


		*)
		#
		# Default to summary
		#
		search_form; sidebar

		cat << EOT
<h2>$(_ 'Summary')</h2>

<form method="get" action="">
<div id="actions">
EOT
		fslink=$(readlink $PKGS_DB/fslink)
		[ -n "$fslink" -a -d "$fslink/$INSTALLED" ] && show_button linkable
		show_button recharge up admin
		cat << EOT
</div>

<article>
<h3>$(_ 'Summary')</h3>
<table class="zebra summary">
<tbody>
<tr>
	<td>$(_ 'Last recharge:')</td>
	<td>$(list=$PKGS_DB/ID
	if [ -e $list ]; then
		ohhmm="$(date +%z)"							# '+0200' for EET (+2 hours)
		offset=$(( 60 * (60 * ${ohhmm:0:3} + ${ohhmm:3:2}) )) # in the seconds
		daynow=$(( ($(date          +%s) + $offset) / 86400 ))
		dayupd=$(( ($(date -r $list +%s) + $offset) / 86400 ))
		days=$(( $daynow - $dayupd ))
		time=$(date -r $list +%R)
		ago="$(_p '%d day ago.' '%d days ago.' $days $days)"
		case $days in
			0) _ 'Today at %s.' $time;;
			1) _ 'Yesterday at %s.' $time;;
			[2-9]) echo $ago;;
			*) echo "<span style='color:red'>$ago</span>"
				_ 'It is recommended to [recharge] the lists.' | \
				sed 's|\[|<a href="?recharge">|;s|\]|</a>|';;
		esac
	else
		_ 'never.'
		_ 'You need to [download] the lists for further work.' | \
		sed 's|\[|<a href="?recharge">|;s|\]|</a>|'
	fi)</td></tr>
<tr>
	<td>$(_ 'Installed packages:')</td>
	<td><a href="?list&amp;my=my&amp;cat=all&amp;repo=Any">
			<b>$(cat $PKGS_DB/installed.info | wc -l)</b>
		</a></td></tr>
<tr>
	<td>$(_ 'Mirrored packages:')</td>
	<td><a href="?list&amp;my=no&amp;cat=all&amp;repo=Any">
		<b>$(cat $PKGS_DB/packages.list | wc -l)</b>
	</a></td></tr>
<tr>
	<td>$(_ 'Upgradeable packages:')</td>
	<td><a href="?up">
			<b>$(cat $PKGS_DB/packages.up | wc -l)</b>
		</a></td></tr>
<tr>
	<td>$(_ 'Installed files:')</td>
	<td><b>$(cat $INSTALLED/*/files.list | wc -l)</b></td></tr>
<tr>
	<td>$(_ 'Blocked packages:')</td>
	<td><a href="?blocked">
			<b>$(cat $PKGS_DB/blocked-packages.list | wc -l)</b>
		</a></td></tr>
</tbody>
</table></article>

<article>
<h3>$(_ 'Latest log entries')</h3>

<pre>
$(tail -n 5 $LOG | fgrep "-" | awk '{print $1, $2, $3, $4, $5, "<a href=\"?info=" $6 "\">" $6 "</a>", $7}')
<a href="index.cgi?file=$LOG">$(_ 'more...')</a>
</pre>
</article>
EOT
		;;
esac

# xHTML 5 footer
export TEXTDOMAIN='tazpkg'
xhtml_footer
exit 0
